// --------------------------------------------------------------------------
//
// GGGears 0.1.3
// Automatic generation of gears geometry, FE-mesh and FE-models
// http://gggears.sourceforge.net/
// Copyright (C) 2010 Konstantinos Poulios
//
// --------------------------------------------------------------------------
//
// This file is part of GGGears.
//
// GGGears is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// --------------------------------------------------------------------------

#include <getfem/getfem_models.h>

using getfem::size_type;
using getfem::scalar_type;

struct fem_body {
  std::string name;
  std::string meshfilename;
  scalar_type lambda;
  scalar_type mu;

  fem_body(std::string name_, std::string meshfilename_,
           scalar_type lambda_, scalar_type mu_)
    : name(name_), meshfilename(meshfilename_), lambda(lambda_), mu(mu_) {};
  fem_body() : name(""), meshfilename(""),
               lambda(scalar_type(0.)), mu(scalar_type(0.)) {};
};

enum { DIRICHLET = 0,
       NEUMANN = 1,
       CONTACT = 2,
       CONTACT_RIGID = 3,
       RIGID_X = 4,
       RIGID_Y = 5,
       RIGID_Z = 6
};

struct fem_BC {
  size_type type;
  std::string name;
  std::string var1, var2;
  std::vector<size_type> rgs1, rgs2;
  std::vector<scalar_type> data_vec;
  std::string data_str;
  size_type id1, id2;

  fem_BC(size_type type_, std::string var1_, std::string var2_,
         std::vector<size_type> rgs1_, std::vector<size_type> rgs2_,
         std::vector<scalar_type> vec_, std::string str_)
    : type(type_), name(""), var1(var1_), var2(var2_), rgs1(rgs1_), rgs2(rgs2_),
      data_vec(vec_), data_str(str_), id1(0), id2(0) {};
  fem_BC(size_type type_)
    : type(type_), name(""), var1(""), var2(""), rgs1(0), rgs2(0),
      data_vec(0), data_str(""), id1(0), id2(0) {};
  fem_BC()
    : type(DIRICHLET), name(""), var1(""), var2(""), rgs1(0), rgs2(0),
      data_vec(0), data_str(""), id1(0), id2(0) {};
};

class fem_model {
   std::vector<fem_body> bodies;
   std::vector<fem_BC> BCs;
   std::vector<getfem::mesh> meshes;
   std::vector<getfem::mesh_im> mims;
   std::vector<getfem::mesh_fem> mfus;
   std::vector<getfem::mesh_fem> mfrhss;
   getfem::model md;
   std::map<std::string,size_type> name2id;
public:
   fem_model() : bodies(0), BCs(0), meshes(0), mims(0), mfus(0), mfrhss(0) {};
   int import_from_file(std::string filename);
   int build();
   int solve();
   int export_results(std::string filename);
};
