#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
 --------------------------------------------------------------------------

 GGGears 0.1.3
 Automatic generation of gears geometry, FE-mesh and FE-models
 http://gggears.sourceforge.net/
 Copyright (C) 2009-2010 Konstantinos Poulios

 --------------------------------------------------------------------------

This file is part of GGGears.

GGGears is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

 --------------------------------------------------------------------------
"""
import re

class compressed_entry(list):
   """Converts a compressed vector of integers to a tuple of integers"""

   def __init__(self, inp_string):
      if type(inp_string) is not str:
         msg = 'A string value was expected. An argument of type \"%s\" has been passed.'
         raise TypeError(msg % type(inp_string))
      content = map(int, expand_compressed_entry(inp_string).split(','))
      list.__init__(self, content)
      self.str = inp_string

   def __str__(self):
      return self.str

def expand_compressed_entry(inp_string):
   """Converts a compressed vector of integers to a simple
      comma separated sequence of integers"""
   operators = re.compile(r'x|\(|\)')
   inp_string = re.sub(r'\s', '', inp_string)  #strip blanks
   while operators.search(inp_string):
      inp_string_old = inp_string
      inp_string = re.sub(r'\B\(([\d+,]+)\)', r'\1' ,inp_string)  # remove unnecessary brackets
      inp_string = re.sub(r'(\d+)x\(([\d+,]+)\)', expand_term ,inp_string)
      inp_string = re.sub(r'(\d+)x(\d+)', expand_term ,inp_string)
      if inp_string == inp_string_old:
         msg = 'The provided string \"%s\" is not a valid compressed entry expression'
         raise ValueError(msg % inp_string)

   return inp_string

def expand_term(match):
   """Expands a term containing a multiplicator operator"""
   return ','.join(int(match.group(1)) * [match.group(2)])

