#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
 --------------------------------------------------------------------------

 GGGears 0.1.3
 Automatic generation of gears geometry, FE-mesh and FE-models
 http://gggears.sourceforge.net/
 Copyright (C) 2010 Konstantinos Poulios

 --------------------------------------------------------------------------

This file is part of GGGears.

GGGears is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

 --------------------------------------------------------------------------
"""

from _tfm import *

class tfm:

   def __init__(self, gear, filename):

      if gear.teeth_number > 0:
         t_no = abs(gear.teeth_number_mesh) # "abs" not really necessary
      else:
         t_no = -abs(gear.teeth_number_mesh) # "abs" not really necessary
      center = [gear.x_offset, gear.y_offset]
      if gear.is_3D:
         center.append(gear.z_offset)
      self.mod = Modification(filename, gear.gear_id, t_no, abs(gear.base_radius),
                              gear.beta_b, gear.tip_diameter/2., center)
      self.is_prepared = False
      self.has_tip_relief = gear.has_tip_relief
      if self.has_tip_relief:
         self.tip_relief_data = \
         (gear.tip_relief_amount_l * 1e-3, gear.tip_relief_amount_r * 1e-3,
          gear.tip_relief_start_radius_l, gear.tip_relief_start_radius_r)
      self.has_profile_crown = gear.has_profile_crown
      if self.has_profile_crown:
         self.profile_crown_data = \
         (gear.profile_crown_amount_l * 1e-3, gear.profile_crown_amount_r * 1e-3)
      self.has_lead_angle = gear.has_lead_angle
      if self.has_lead_angle:
         self.lead_angle_data = \
         (gear.lead_angle_amount_l * 1e-3, gear.lead_angle_amount_r * 1e-3)
      self.has_end_relief = gear.has_end_relief
      if self.has_end_relief:
         self.end_relief_data = \
         (gear.end_relief_amount_l * 1e-3, gear.end_relief_amount_r * 1e-3,
          gear.end_relief_length_l, gear.end_relief_length_r)
      self.has_lead_crown = gear.has_lead_crown
      if self.has_lead_crown:
         self.lead_crown_data = \
         (gear.lead_crown_amount_l * 1e-3, gear.lead_crown_amount_r * 1e-3)

   def export_mesh(self, filename):

      if not self.is_prepared:
         self.mod.prepare()
         self.is_prepared = False
      if self.has_tip_relief:
         self.mod.tip_relief(*self.tip_relief_data)
      if self.has_profile_crown:
         self.mod.profile_crown(*self.profile_crown_data)
      if self.has_lead_angle:
         self.mod.lead_angle(*self.lead_angle_data)
      if self.has_end_relief:
         self.mod.end_relief(*self.end_relief_data)
      if self.has_lead_crown:
         self.mod.lead_crown(*self.lead_crown_data)
      self.mod.export_mesh(filename)

