#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
 --------------------------------------------------------------------------

 GGGears 0.1.3
 Automatic generation of gears geometry, FE-mesh and FE-models
 http://gggears.sourceforge.net/
 Copyright (C) 2010 Konstantinos Poulios

 --------------------------------------------------------------------------

This file is part of GGGears.

GGGears is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

 --------------------------------------------------------------------------
"""
from sys import path
path.append('/usr/lib/paraview')
try:
   from paraview.simple import *
   has_paraview = True
except:
   has_paraview = False
   
path.pop()

def create_post_proc_view(varnames, basename):

   def GetProxyToDelete():
      it = servermanager.vtkSMProxyIterator()
      it.Begin()
      while not it.IsAtEnd():
        if it.GetGroup().find("prototypes") != -1:
           it.Next()
           continue
        proxy = servermanager._getPyProxy(it.GetProxy())
        proxygroup = it.GetGroup()
        if proxygroup != 'timekeeper' and proxy != None \
        and proxygroup.find("pq_helper_proxies") == -1 :
            return proxy
        it.Next()
      return None

   if not has_paraview:
      return 1

   pxm = servermanager.ProxyManager()

   viewVM = servermanager.CreateRenderView()
   viewVM.Background = [0.5, 0.5, 0.5]
   pxm.RegisterProxy('views', 'View von Mises stresses', viewVM)
   viewNCF = servermanager.CreateRenderView()
   viewNCF.Background = [0.5, 0.5, 0.5]
   pxm.RegisterProxy('views', 'View normal contact forces', viewNCF)

   lutVM = GetLookupTableForArray("Von_Mises_Stresses", 1,
                                  RGBPoints = [0, 0, 0, 1, 1., 1, 0, 0],
                                  ColorSpace = "HSV",
                                  LockScalarRange = 1)
   lutNCF = GetLookupTableForArray("Normal_Contact_Forces", 3,
                                   RGBPoints = [0, 0, 0, 1, 1., 1, 0, 0],
                                   ColorSpace = "HSV",
                                   VectorMode = "Magnitude")

   maxval = 0.
   for var in varnames:
      fname = '%s_%s.vtk' % (basename, var)
      source = LegacyVTKReader(FileNames=fname)
      RenameSource('Undeformed %s' % var, source)

      hist = Histogram(source, BinCount = 15, CalculateAverages = 0)
      hist_table = servermanager.Fetch(hist)
      meanvals = []
      populations = []
      for i in range(hist_table.GetNumberOfRows()):
         meanvals.append(hist_table.GetColumn(0).GetValue(i))
         populations.append(hist_table.GetColumn(1).GetValue(i))
      tot_population = sum(populations)
      limit_population = 0.05 * tot_population / hist.BinCount
      while populations.pop() < limit_population:
         meanvals.pop()
      maxval = max(maxval, meanvals[-1])
      Delete(hist)

      warp = WarpByVector(Input=source)
      RenameSource('Deformed %s' % var, warp)

      rep1 = servermanager.CreateRepresentation(source, viewVM)
      rep1.Representation = 'Wireframe'
      rep1.Opacity = 0.3
      rep1.Visibility = 0
#      rep1.ColorArrayName = 'Von_Mises_Stresses_%s' % var
#      rep1.LookupTable = lutVM
      pxm.RegisterProxy('representations',
                        'Undeformed %s' % var, rep1)

      rep2 = servermanager.CreateRepresentation(warp, viewVM)
      rep2.Representation = 'Surface With Edges'
      rep2.ColorArrayName = 'Von_Mises_Stresses_%s' % var
      rep2.EdgeColor = [0., 0., 0.]
      rep2.LineWidth = 0.5
      rep2.LookupTable = lutVM
      pxm.RegisterProxy('representations',
                        'Deformed %s with von Mises stresses' % var, rep2)

      rep3 = servermanager.CreateRepresentation(warp, viewNCF)
      rep3.Representation = 'Wireframe'
      rep3.ColorArrayName = 'Normal_Contact_Forces_on_%s' % var
      rep3.EdgeColor = [0., 0., 0.]
      rep3.LineWidth = 0.5
      rep3.LookupTable = lutNCF
      pxm.RegisterProxy('representations',
                        'Deformed %s with normal contact forces' % var, rep3)

   lutVM.RGBPoints[4] = maxval
   viewVM.ResetCamera()
   viewNCF.ResetCamera()
   servermanager.SaveState('%s.pvsm' % basename)

   tobedeleted = GetProxyToDelete()
   while tobedeleted:
      Delete(tobedeleted)
      tobedeleted = GetProxyToDelete()

