#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
 --------------------------------------------------------------------------

 GGGears 0.1.3
 Automatic generation of gears geometry, FE-mesh and FE-models
 http://gggears.sourceforge.net/
 Copyright (C) 2010 Konstantinos Poulios

 --------------------------------------------------------------------------

This file is part of GGGears.

GGGears is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

 --------------------------------------------------------------------------
"""
import sys
try:
    import pygtk
    pygtk.require('2.0')
    import gtk
    assert gtk.gtk_version >= (2, 10, 0)
    assert gtk.pygtk_version >= (2, 10, 0)
except:
    print('PyGTK version 2.10.0 or higher is required to run this program.')
    sys.exit(1)

import pango
from math import sqrt


class preview_tfm:
   """This class contains a backing pixmap for the preview of tooth flank
      modifications and all necessary data and methods to redraw it"""

   def __init__(self, area, x_min, x_max, scale):
      self.resize(area)
      self.x_min = x_min
      self.x_max = x_max
      self.scale = scale
      self.scale_lim = scale
      self.clear()

   def clear(self):
      self.mods = []

   def resize(self, area):
      x, y, width, height = area.get_allocation()
      self.pixmap = gtk.gdk.Pixmap(area.window, width, height)
      self.pango_layout = area.create_pango_layout('')
      self.pango_layout.set_font_description(pango.FontDescription('Sans 9'))

      self.style = area.get_style()
      self.pixmap.draw_rectangle(self.style.white_gc,True, 0, 0, width, height)
      self.width, self.height = self.pixmap.get_size()
      self.x0 = int(0.15 * self.width)
      self.y0 = int(0.85 * self.height)
      self.x1 = int(0.65 * self.width)
      self.y1 = int(0.1 * self.height)
      self.dx = self.x1 - self.x0
      self.dy = self.y1 - self.y0

   def draw_curves(self, scale_lim):
      nn = 20
      vec_val_L = (nn+1)*[0]
      vec_val_R = (nn+1)*[0]
      for cur_vec_val_L, cur_vec_val_R in self.mods:
         for i in range(nn+1):
            vec_val_L[i] += cur_vec_val_L[i]
            vec_val_R[i] += cur_vec_val_R[i]
      max_val = max(max(map(abs, vec_val_L)), max(map(abs, vec_val_R)))
      if max_val > 1e-5:
         self.scale_lim = 60. / max_val
      else:
         self.scale_lim = 1e5
      while self.scale > min(self.scale_lim, scale_lim):
         self.scale = self.scale / 2
      while 2 * self.scale < min(self.scale_lim, scale_lim):
         self.scale = 2 * self.scale
      for i in range(nn+1):
         vec_val_L[i] = int(self.scale * vec_val_L[i])
         vec_val_R[i] = int(self.scale * vec_val_R[i])

      x0 = self.x0;      y0 = self.y0
      x1 = self.x1;
      dy = self.dy / nn

      self.pixmap.draw_rectangle(self.style.white_gc,True,
                                 0, 0, self.width, self.height)
      points_l = []
      points_r = []
      for i in range(nn+1):
         points_l.append((x0 + vec_val_L[i], y0 + int(i * dy)))
         points_r.append((x1 - vec_val_R[i], y0 + int(i * dy)))
      self.pixmap.draw_lines(self.style.black_gc, points_l)
      self.pixmap.draw_lines(self.style.black_gc, points_r)

   def draw_profile(self, scale_lim):
      self.draw_curves(scale_lim)
      tip_text = 'TIP'
      root_text = 'ROOT'
      tip_val = 'r=%.1f' % abs(self.x_max)
      root_val = 'r=%.1f' % abs(self.x_min)
      self.draw_annotation(tip_text, root_text, tip_val, root_val)

   def draw_lead(self, scale_lim):
      self.draw_curves(scale_lim)
      end_text = 'END'
      start_text = 'START'
      end_val = 'x=%.1f' % self.x_max
      start_val = 'x=%.1f' % self.x_min
      self.draw_annotation(end_text, start_text, end_val, start_val)

   def draw_annotation(self, lim1, lim2, val1, val2):
      x0 = self.x0;      y0 = self.y0
      x1 = self.x1;      y1 = self.y1
      self.pango_layout.set_text(lim1)
      w, h = self.pango_layout.get_pixel_size()
      self.pixmap.draw_layout(self.style.black_gc,
                              (x0 + x1 - w)/2, (y1 - h)/2, self.pango_layout)
      self.pango_layout.set_text(lim2)
      w, h = self.pango_layout.get_pixel_size()
      self.pixmap.draw_layout(self.style.black_gc,
                              (x0 + x1 - w)/2, (y0 + self.height - h)/2,
                              self.pango_layout)
      self.pango_layout.set_text(val1)
      w, h = self.pango_layout.get_pixel_size()
      self.pixmap.draw_layout(self.style.black_gc,
                              x1 + 20, y1 - h/2, self.pango_layout)
      self.pango_layout.set_text(val2)
      w, h = self.pango_layout.get_pixel_size()
      self.pixmap.draw_layout(self.style.black_gc,
                              x1 + 20, y0 - h/2, self.pango_layout)
      self.pango_layout.set_text('L')
      w, h = self.pango_layout.get_pixel_size()
      self.pixmap.draw_layout(self.style.black_gc,
                              x0, (y1 - h)/2 + 5, self.pango_layout)
      self.pango_layout.set_text('R')
      w, h = self.pango_layout.get_pixel_size()
      self.pixmap.draw_layout(self.style.black_gc,
                              x1 - w, (y1 - h)/2 + 5, self.pango_layout)
      scala_length = 30. / self.scale
      scala_length = 5. * int(scala_length/5.)
      xx1 = 5
      xx2 = xx1 + int(self.scale * scala_length)
      yy = self.height - 5
      self.pixmap.draw_line(self.style.black_gc, xx1, yy-2, xx1, yy+2)
      self.pixmap.draw_line(self.style.black_gc, xx1,   yy, xx2,   yy)
      self.pixmap.draw_line(self.style.black_gc, xx2, yy-2, xx2, yy+2)
      self.pango_layout.set_text('%.0f' % scala_length)
      w, h = self.pango_layout.get_pixel_size()
      self.pixmap.draw_layout(self.style.black_gc,
                              (xx1 + xx2 - w)/2, yy - h - 2, self.pango_layout)

   def add_tip_relief(self, val_L, val_R, start_r_L, start_r_R, r_b):
      """Calculates the contour of a tip relief tooth flank modification and
         appends it to the current list of modifications"""
      s_min = sqrt(self.x_min**2 - r_b**2)
      s_max = sqrt(self.x_max**2 - r_b**2)
      if abs(start_r_L) > abs(r_b):
         s_L = sqrt(start_r_L**2 - r_b**2)
      else:
         s_L = 0.
      if abs(start_r_R) > abs(r_b):
         s_R = sqrt(start_r_R**2 - r_b**2)
      else:
         s_R = 0.
      if s_min > s_max: # for internal gearings
         s_min = -s_min
         s_max = -s_max
         s_L = -s_L
         s_R = -s_R
      if s_L < s_min: s_L = s_max + 1.
      if s_R < s_min: s_R = s_max + 1.
      nn = 20
      ds = (s_max - s_min) / nn
      vec_val_L = (nn+1)*[0.]
      vec_val_R = (nn+1)*[0.]
      for i in range(nn+1):
         s = s_min + i*ds
         if s > s_L:
            vec_val_L[i] = val_L*(s-s_L)/(s_max-s_L)
         if s > s_R:
            vec_val_R[i] = val_R*(s-s_R)/(s_max-s_R)
      self.mods.append((vec_val_L,vec_val_R))

   def add_profile_crown(self, val_L, val_R, r_b):
      """Calculates the contour of a profile crowning tooth flank modification and
         appends it to the current list of modifications"""
      s_min = sqrt(self.x_min**2 - r_b**2)
      s_max = sqrt(self.x_max**2 - r_b**2)
      sM = (s_min + s_max)/2
      nn = 20
      ds = (s_max - s_min) / nn
      vec_val_L = (nn+1)*[0.]
      vec_val_R = (nn+1)*[0.]
      for i in range(nn+1):
         s = s_min + i*ds
         ss = 2 * (s-sM)/(s_max-s_min)
         vec_val_L[i] = val_L*ss**2
         vec_val_R[i] = val_R*ss**2
      self.mods.append((vec_val_L,vec_val_R))

   def add_lead_angle(self, val_L, val_R):
      """Calculates the contour of a lead angle tooth flank modification and
         appends it to the current list of modifications"""
      if val_L < 0:
         x0L = self.x_min
      else:
         x0L = self.x_max
      if val_R > 0:
         x0R = self.x_min
      else:
         x0R = self.x_max
      dxL = self.x_min - self.x_max
      dxR = self.x_max - self.x_min
      valid_L = (abs(dxL) > 1e-6)
      valid_R = (abs(dxR) > 1e-6)
      nn = 20
      dx = (self.x_max - self.x_min) / nn
      vec_val_L = (nn+1)*[0.]
      vec_val_R = (nn+1)*[0.]
      for i in range(nn+1):
         x = self.x_min + i*dx
         if valid_L: vec_val_L[i] = val_L*(x-x0L)/dxL
         if valid_R: vec_val_R[i] = val_R*(x-x0R)/dxR
      self.mods.append((vec_val_L,vec_val_R))

   def add_end_relief(self, val_L, val_R, length_L, length_R):
      """Calculates the contour of a end relief tooth flank modification and
         appends it to the current list of modifications"""
      if length_L > 0. and length_L < (self.x_max - self.x_min)/2:
         x_minL = self.x_min + length_L
         x_maxL = self.x_max - length_L
      else:
         x_minL = self.x_min - 1.
         x_maxL = self.x_max + 1.
      if length_R > 0. and length_R < (self.x_max - self.x_min)/2:
         x_minR = self.x_min + length_R
         x_maxR = self.x_max - length_R
      else:
         x_minR = self.x_min - 1.
         x_maxR = self.x_max + 1.
      nn = 20
      dx = (self.x_max - self.x_min) / nn
      vec_val_L = (nn+1)*[0.]
      vec_val_R = (nn+1)*[0.]
      for i in range(nn+1):
         x = self.x_min + i*dx
         if x < x_minL:
            vec_val_L[i] = val_L*(x_minL-x)/length_L
         elif x > x_maxL:
            vec_val_L[i] = val_L*(x-x_maxL)/length_L
         if x < x_minR:
            vec_val_R[i] = val_R*(x_minR-x)/length_R
         elif x > x_maxR:
            vec_val_R[i] = val_R*(x-x_maxR)/length_R
      self.mods.append((vec_val_L,vec_val_R))

   def add_lead_crown(self, val_L, val_R):
      """Calculates the contour of a lead crowning tooth flank modification and
         appends it to the current list of modifications"""
      xM = (self.x_min + self.x_max)/2 # this should always be equal zero
      nn = 20
      dx = (self.x_max - self.x_min) / nn
      vec_val_L = (nn+1)*[0.]
      vec_val_R = (nn+1)*[0.]
      for i in range(nn+1):
         x = self.x_min + i*dx
         xx = 2 * (x-xM)/(self.x_max-self.x_min)
         vec_val_L[i] = val_L*xx**2
         vec_val_R[i] = val_R*xx**2
      self.mods.append((vec_val_L,vec_val_R))


